// Tony Urschler
// October 11, 2021

#include "liblicense.h"

// waits for a license to become available
int getlicense(int semid, int* nlicenses) {
    // wait for a license to open up
    struct sembuf sem_op;
    sem_op.sem_num = 0;
    sem_op.sem_op = -1;
    sem_op.sem_flg = 0;
    semop(semid, &sem_op, 1);

    // once you get a license, decrement and return
    *nlicenses -= 1;
    return 0;
}

// returns a license
int returnlicense(int semid, int* nlicenses) {
    // signal semaphore, and add to licenses
    struct sembuf sem_op;
    sem_op.sem_num = 0;
    sem_op.sem_op = 1;
    sem_op.sem_flg = 0;
    semop(semid, &sem_op, 1);
    
    *nlicenses += 1;
    return 0;
}

// initializes semaphore with appropriate value
// return -1 if creation failed
int initlicense(int val, int semids[2]) {
    // create semaphore
    semids[0] = semget(IPC_PRIVATE, 1, IPC_CREAT | 0600);
    semids[1] = semget(IPC_PRIVATE, 1, IPC_CREAT | 0600);
    if (semids[0] == -1 || semids[1] == -1)
        return -1;
    
    int rc;
    union semun sem_val;

    // set semaphore value
    sem_val.val = val;
    rc = semctl(semids[0], 0, SETVAL, sem_val);
    if (rc ==-1)
        return -1;
    sem_val.val = 1;
    rc = semctl(semids[1], 0, SETVAL, sem_val);
    if (rc ==-1)
        return -1;

    return 0;
}

// adds n amount of licenses
int addtolicenses(int n, int semid, int* nlicenses) {
    struct sembuf sem_op;
    sem_op.sem_num = 0;
    sem_op.sem_op = n;
    sem_op.sem_flg = 0;
    semop(semid, &sem_op, 1);
    
    *nlicenses += n;    
    
    return 0;
}

// removes n amount of licenses
int removelicenses(int n, int semid, int* nlicenses) {
    struct sembuf sem_op;
    sem_op.sem_num = 0;
    sem_op.sem_op = -n;
    sem_op.sem_flg = 0;
    semop(semid, &sem_op, 1);
    
    *nlicenses -= n;    
    
    return 0;
}

// logs msg to file
int logmsg(int semid, const char* msg) {
    
    // wait for access
    struct sembuf sem_op;
    sem_op.sem_num = 0;
    sem_op.sem_op = -1;
    sem_op.sem_flg = 0;
    semop(semid, &sem_op, 1);

    // critical section
    FILE* fptr;
    fptr = fopen("logfile", "a");
    fprintf(fptr, "%s\n", msg);
    fclose(fptr);

    // return control
    sem_op.sem_num = 0;
    sem_op.sem_op = 1;
    sem_op.sem_flg = 0;
    semop(semid, &sem_op, 1);
    
    return 0;
}


